@@ -14,4 +14,4 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-__all__ = ['schema']
+__all__ = ["schema"]
