@@ -58,13 +58,13 @@ SCHEMA_RESERVED_PROPS = (
 # MappingDataType = Dict[str, Union[PropType, List[PropsType]]]
 # was: Union[str, MappingDataType, List[MappingDataType]]
 JsonDataType = Any
-AtomicPropType = Union[str, int, bool, "Schema", List[str], List["Field"]]
+AtomicPropType = Union[None, str, int, float, bool, "Schema", List[str], List["Field"]]
 PropType = Union[
     AtomicPropType,
-    Dict[str, Optional[AtomicPropType]],
-    List[Dict[str, Optional[AtomicPropType]]],
+    Dict[str, AtomicPropType],
+    List[Dict[str, AtomicPropType]],
 ]
-PropsType = Dict[str, Optional[PropType]]
+PropsType = Dict[str, PropType]
 
 FIELD_RESERVED_PROPS = ("default", "name", "doc", "order", "type")
 
@@ -95,7 +95,7 @@ class Schema:
         # Ensure valid ctor args
         if not isinstance(atype, str):
             raise SchemaParseException(
-                "Schema type '{}' must be a string, was '{}.".format(atype, type(atype))
+                f"Schema type '{atype}' must be a string, was '{type(atype)}."
             )
         elif atype not in VALID_TYPES:
             fail_msg = f"{atype} is not a valid type."
@@ -216,8 +216,10 @@ class Names:
         elif to_add.fullname in self.names:
             fail_msg = f'The name "{to_add.fullname}" is already in use.'
             raise SchemaParseException(fail_msg)
+        elif to_add.fullname is None:
+            fail_msg = f"{to_add.fullname} is missing, but this is impossible."
+            raise SchemaParseException(fail_msg)
 
-        assert to_add.fullname
         self.names[to_add.fullname] = new_schema
         return to_add
 
@@ -301,8 +303,8 @@ class Field:
                 type_schema = make_avsc_object(atype, names)
             except Exception as e:
                 raise SchemaParseException(
-                    f'Type property "{atype}" not a valid Avro schema: {e}'
-                )
+                    f'Type property "{atype}" not a valid Avro schema.'
+                ) from e
         self.set_prop("type", type_schema)
         self.set_prop("name", name)
         self.type = type_schema
@@ -407,9 +409,9 @@ class ArraySchema(Schema):
                 items_schema = make_avsc_object(items, names)
             except Exception as err:
                 raise SchemaParseException(
-                    "Items schema ({}) not a valid Avro schema: {} (known "
-                    "names: {})".format(items, err, list(names.names.keys()))
-                )
+                    f"Items schema ({items}) not a valid Avro schema: (known "
+                    f"names: {list(names.names.keys())})."
+                ) from err
 
         self.set_prop("items", items_schema)
 
@@ -449,8 +451,8 @@ class UnionSchema(Schema):
                     new_schema = make_avsc_object(schema, names)
                 except Exception as err:
                     raise SchemaParseException(
-                        f"Union item must be a valid Avro schema: {err}. {schema}"
-                    )
+                        f"Union item must be a valid Avro schema: {schema}"
+                    ) from err
             # check the new schema
             if (
                 new_schema.type in VALID_TYPES
@@ -576,12 +578,14 @@ def make_avsc_object(json_data: JsonDataType, names: Optional[Names] = None) ->
     """
     if names is None:
         names = Names()
-    assert isinstance(names, Names)
 
-    if isinstance(json_data, Dict) and json_data.get("name") == "Any":
-        del names.names["Any"]
-    elif not names.has_name("Any", None):
-        EnumSchema("Any", None, ["Any"], names=names)
+    if (
+        isinstance(json_data, Dict)
+        and json_data.get("name") == "org.w3id.cwl.salad.Any"
+    ):
+        del names.names["org.w3id.cwl.salad.Any"]
+    elif not names.has_name("org.w3id.cwl.salad.Any", None):
+        EnumSchema("org.w3id.cwl.salad.Any", None, ["Any"], names=names)
 
     # JSON object (non-union)
     if isinstance(json_data, dict):
@@ -622,7 +626,6 @@ def make_avsc_object(json_data: JsonDataType, names: Optional[Names] = None) ->
                     symbols = cast(List[str], symbols)
                 return EnumSchema(name, namespace, symbols, names, doc, other_props)
             if atype in ["record", "error"]:
-                assert isinstance(atype, str)
                 fields = json_data.get("fields")
                 if not isinstance(fields, list):
                     raise SchemaParseException(
