@@ -11,16 +11,21 @@ from typing import (
     TextIO,
     Union,
 )
+from urllib.parse import urlsplit
+
 
 from . import schema
 from .codegen_base import CodeGenBase
 from .exceptions import SchemaSaladException
 from .java_codegen import JavaCodeGen
 from .python_codegen import PythonCodeGen
+from .typescript_codegen import TypeScriptCodeGen
 from .ref_resolver import Loader
 from .schema import shortname
 from .utils import aslist
 
+FIELD_SORT_ORDER = ["id", "class", "name"]
+
 
 def codegen(
     lang: str,
@@ -29,12 +34,25 @@ def codegen(
     loader: Loader,
     target: Optional[str] = None,
     examples: Optional[str] = None,
+    package: Optional[str] = None,
+    copyright: Optional[str] = None,
+    parser_info: Optional[str] = None,
 ) -> None:
     """Generate classes with loaders for the given Schema Salad description."""
 
     j = schema.extend_and_specialize(i, loader)
 
     gen = None  # type: Optional[CodeGenBase]
+    base = schema_metadata.get("$base", schema_metadata.get("id"))
+    sp = urlsplit(base)
+    pkg = (
+        package
+        if package
+        else ".".join(
+            list(reversed(sp.netloc.split("."))) + sp.path.strip("/").split("/")
+        )
+    )
+    info = parser_info or pkg
     if lang == "python":
         if target:
             dest: Union[TextIOWrapper, TextIO] = open(
@@ -42,16 +60,20 @@ def codegen(
             )
         else:
             dest = sys.stdout
-        gen = PythonCodeGen(dest)
+
+        gen = PythonCodeGen(dest, copyright=copyright, parser_info=info)
     elif lang == "java":
         gen = JavaCodeGen(
-            schema_metadata.get("$base", schema_metadata.get("id")),
+            base,
             target=target,
             examples=examples,
+            package=pkg,
+            copyright=copyright,
         )
+    elif lang == "typescript":
+        gen = TypeScriptCodeGen(base, target=target, package=pkg, examples=examples)
     else:
         raise SchemaSaladException(f"Unsupported code generation language '{lang}'")
-    assert gen is not None
 
     gen.prologue()
 
@@ -99,7 +121,14 @@ def codegen(
             )
             gen.add_vocab(shortname(rec["name"]), rec["name"])
 
-            for field in rec.get("fields", []):
+            sorted_fields = sorted(
+                rec.get("fields", []),
+                key=lambda i: FIELD_SORT_ORDER.index(i["name"].split("/")[-1])
+                if i["name"].split("/")[-1] in FIELD_SORT_ORDER
+                else 100,
+            )
+
+            for field in sorted_fields:
                 if field.get("jsonldPredicate") == "@id":
                     subscope = field.get("subscope")
                     fieldpred = field["name"]
@@ -112,7 +141,7 @@ def codegen(
                     )
                     break
 
-            for field in rec.get("fields", []):
+            for field in sorted_fields:
                 optional = bool("https://w3id.org/cwl/salad#null" in field["type"])
                 type_loader = gen.type_loader(field["type"])
                 jld = field.get("jsonldPredicate")
