@@ -1,53 +1,27 @@
 """Base class for the generation of loaders from schema-salad definitions."""
-import collections
-from typing import Any, Dict, List, MutableSequence, Optional, Set, Union
-
-from . import schema
+from collections import OrderedDict
+from typing import Any, Dict, List, MutableSequence, NamedTuple, Optional, Set, Union
 
 
-class TypeDef:  # pylint: disable=too-few-public-methods
+class TypeDef(NamedTuple):
     """Schema Salad type description."""
 
-    __slots__ = [
-        "name",
-        "init",
-        "is_uri",
-        "scoped_id",
-        "ref_scope",
-        "loader_type",
-        "instance_type",
-    ]
-
-    # switch to class-style typing.NamedTuple once support for Python < 3.6
-    # is dropped
-    def __init__(
-        self,  # pylint: disable=too-many-arguments
-        name: str,
-        init: str,
-        is_uri: bool = False,
-        scoped_id: bool = False,
-        ref_scope: Optional[int] = 0,
-        loader_type: Optional[str] = None,
-        instance_type: Optional[str] = None,
-    ) -> None:
-        self.name = name
-        self.init = init
-        self.is_uri = is_uri
-        self.scoped_id = scoped_id
-        self.ref_scope = ref_scope
-        # Follow attributes used by Java but not Python.
-        self.loader_type = loader_type
-        self.instance_type = instance_type
+    name: str
+    init: str
+    is_uri: bool = False
+    scoped_id: bool = False
+    ref_scope: Optional[int] = 0
+    loader_type: Optional[str] = None
+    instance_type: Optional[str] = None
+    abstract: bool = False
 
 
 class CodeGenBase:
     """Abstract base class for schema salad code generators."""
 
     def __init__(self) -> None:
-        self.collected_types = (
-            collections.OrderedDict()
-        )  # type: collections.OrderedDict[str, TypeDef]
-        self.vocab = {}  # type: Dict[str, str]
+        self.collected_types: OrderedDict[str, TypeDef] = OrderedDict()
+        self.vocab: Dict[str, str] = {}
 
     def declare_type(self, declared_type: TypeDef) -> TypeDef:
         """Add this type to our collection, if needed."""
@@ -66,7 +40,7 @@ class CodeGenBase:
     @staticmethod
     def safe_name(name: str) -> str:
         """Generate a safe version of the given name."""
-        return schema.avro_name(name)
+        raise NotImplementedError()
 
     def begin_class(
         self,  # pylint: disable=too-many-arguments
@@ -137,5 +111,5 @@ class CodeGenBase:
         raise NotImplementedError()
 
     def secondaryfilesdsl_loader(self, inner: TypeDef) -> TypeDef:
-        """Construct the TypeDef for secondary files"""
+        """Construct the TypeDef for secondary files."""
         raise NotImplementedError()
