@@ -1,3 +1,5 @@
+"""Resource fetching."""
+import logging
 import os
 import re
 import sys
@@ -10,6 +12,7 @@ from .exceptions import ValidationException
 from .utils import CacheType
 
 _re_drive = re.compile(r"/([a-zA-Z]):")
+_logger = logging.getLogger("salad")
 
 
 class Fetcher:
@@ -20,7 +23,7 @@ class Fetcher:
     ) -> None:
         pass
 
-    def fetch_text(self, url: str) -> str:
+    def fetch_text(self, url: str, content_types: Optional[List[str]] = None) -> str:
         raise NotImplementedError()
 
     def check_exists(self, url: str) -> bool:
@@ -44,12 +47,10 @@ class DefaultFetcher(Fetcher):
         self.cache = cache
         self.session = session
 
-    def fetch_text(self, url: str) -> str:
-        if url in self.cache and self.cache[url] is not True:
-            # treat "True" as a placeholder that indicates something exists but
-            # not necessarily what its contents is.
-            result = self.cache[url]
-            assert isinstance(result, str)
+    def fetch_text(self, url: str, content_types: Optional[List[str]] = None) -> str:
+        """Retrieve the given resource as a string."""
+        result = self.cache.get(url, None)
+        if isinstance(result, str):
             return result
 
         split = urllib.parse.urlsplit(url)
@@ -57,10 +58,20 @@ class DefaultFetcher(Fetcher):
 
         if scheme in ["http", "https"] and self.session is not None:
             try:
-                resp = self.session.get(url)
+                headers = {}
+                if content_types:
+                    headers["Accept"] = ", ".join(content_types) + ", */*;q=0.8"
+                resp = self.session.get(url, headers=headers)
                 resp.raise_for_status()
             except Exception as e:
                 raise ValidationException(f"Error fetching {url}: {e}") from e
+            if content_types and "content-type" in resp.headers:
+                content_type = resp.headers["content-type"].split(";")[:1][0]
+                if content_type not in content_types:
+                    _logger.warning(
+                        f"While fetching {url}, got content-type of "
+                        f"'{content_type}'. Expected one of {content_types}."
+                    )
             return resp.text
         if scheme == "file":
             try:
@@ -90,9 +101,11 @@ class DefaultFetcher(Fetcher):
         split = urllib.parse.urlsplit(url)
         scheme, path = split.scheme, split.path
 
-        if scheme in ["http", "https"] and self.session is not None:
+        if scheme in ["http", "https"]:
+            if self.session is None:
+                raise ValidationException(f"Can't check {scheme} URL, session is None")
             try:
-                resp = self.session.head(url)
+                resp = self.session.head(url, allow_redirects=True)
                 resp.raise_for_status()
             except Exception:
                 return False
@@ -102,7 +115,7 @@ class DefaultFetcher(Fetcher):
             return os.path.exists(urllib.request.url2pathname(str(path)))
         if scheme == "mailto":
             return True
-        raise ValidationException(f"Unsupported scheme in url: {url}")
+        raise ValidationException(f"Unsupported scheme '{scheme}' in url: {url}")
 
     def urljoin(self, base_url: str, url: str) -> str:
         if url.startswith("_:"):
