@@ -3,7 +3,7 @@ import os
 import shutil
 import string
 from io import StringIO
-from io import open as io_open
+from pathlib import Path
 from typing import (
     Any,
     Dict,
@@ -14,11 +14,10 @@ from typing import (
     Set,
     Union,
 )
-from urllib.parse import urlsplit
 
 import pkg_resources
 
-from . import schema
+from . import _logger, schema
 from .codegen_base import CodeGenBase, TypeDef
 from .exceptions import SchemaException
 from .schema import shortname
@@ -30,10 +29,10 @@ from .schema import shortname
 USE_ONE_OR_LIST_OF_TYPES = False
 
 
-def _ensure_directory_and_write(path: str, contents: str) -> None:
-    dirname = os.path.dirname(path)
-    _safe_makedirs(dirname)
-    with io_open(path, mode="w", encoding="utf-8") as f:
+def _ensure_directory_and_write(path: Path, contents: str) -> None:
+    _safe_makedirs(path.parent)
+    with open(path, mode="w", encoding="utf-8") as f:
+        _logger.info("Writing file: %s", path)
         f.write(contents)
 
 
@@ -48,88 +47,103 @@ def doc_to_doc_string(doc: Optional[str], indent_level: int = 0) -> str:
     return doc_str
 
 
-def _safe_makedirs(path):
-    # type: (str) -> None
-    if not os.path.exists(path):
+def _safe_makedirs(path: Path) -> None:
+    if not path.exists():
         os.makedirs(path)
+        _logger.info("Created directory: %s", path)
 
 
+_string_type_def = TypeDef(
+    instance_type="String",
+    init="new PrimitiveLoader<String>(String.class)",
+    name="StringInstance",
+    loader_type="Loader<String>",
+)
+
+_int_type_def = TypeDef(
+    instance_type="Integer",
+    init="new PrimitiveLoader<Integer>(Integer.class)",
+    name="IntegerInstance",
+    loader_type="Loader<Integer>",
+)
+
+_long_type_def = TypeDef(
+    instance_type="Long",
+    name="LongInstance",
+    loader_type="Loader<Long>",
+    init="new PrimitiveLoader<Long>(Long.class)",
+)
+
+_float_type_def = TypeDef(
+    instance_type="Double",
+    name="DoubleInstance",
+    loader_type="Loader<Double>",
+    init="new PrimitiveLoader<Double>(Double.class)",
+)
+
+_bool_type_def = TypeDef(
+    instance_type="Boolean",
+    name="BooleanInstance",
+    loader_type="Loader<Boolean>",
+    init="new PrimitiveLoader<Boolean>(Boolean.class)",
+)
+
+_null_type_def = TypeDef(
+    instance_type="Object",
+    name="NullInstance",
+    loader_type="Loader<Object>",
+    init="new NullLoader()",
+)
+
+_any_type_def = TypeDef(
+    instance_type="Object",
+    name="AnyInstance",
+    init="new AnyLoader()",
+    loader_type="Loader<Object>",
+)
+
 prims = {
-    "http://www.w3.org/2001/XMLSchema#string": TypeDef(
-        instance_type="String",
-        init="new PrimitiveLoader<String>(String.class)",
-        name="StringInstance",
-        loader_type="Loader<String>",
-    ),
-    "http://www.w3.org/2001/XMLSchema#int": TypeDef(
-        instance_type="Integer",
-        init="new PrimitiveLoader<Integer>(Integer.class)",
-        name="IntegerInstance",
-        loader_type="Loader<Integer>",
-    ),
-    "http://www.w3.org/2001/XMLSchema#long": TypeDef(
-        instance_type="Long",
-        name="LongInstance",
-        loader_type="Loader<Long>",
-        init="new PrimitiveLoader<Long>(Long.class)",
-    ),
-    "http://www.w3.org/2001/XMLSchema#float": TypeDef(
-        instance_type="Double",
-        name="DoubleInstance",
-        loader_type="Loader<Double>",
-        init="new PrimitiveLoader<Double>(Double.class)",
-    ),
-    "http://www.w3.org/2001/XMLSchema#double": TypeDef(
-        instance_type="Double",
-        name="DoubleInstance",
-        loader_type="Loader<Double>",
-        init="new PrimitiveLoader<Double>(Double.class)",
-    ),
-    "http://www.w3.org/2001/XMLSchema#boolean": TypeDef(
-        instance_type="Boolean",
-        name="BooleanInstance",
-        loader_type="Loader<Boolean>",
-        init="new PrimitiveLoader<Boolean>(Boolean.class)",
-    ),
-    "https://w3id.org/cwl/salad#null": TypeDef(
-        instance_type="Object",
-        name="NullInstance",
-        loader_type="Loader<Object>",
-        init="new NullLoader()",
-    ),
-    "https://w3id.org/cwl/salad#Any": TypeDef(
-        instance_type="Object",
-        name="AnyInstance",
-        init="new AnyLoader()",
-        loader_type="Loader<Object>",
-    ),
+    "http://www.w3.org/2001/XMLSchema#string": _string_type_def,
+    "http://www.w3.org/2001/XMLSchema#int": _int_type_def,
+    "http://www.w3.org/2001/XMLSchema#long": _long_type_def,
+    "http://www.w3.org/2001/XMLSchema#float": _float_type_def,
+    "http://www.w3.org/2001/XMLSchema#double": _float_type_def,
+    "http://www.w3.org/2001/XMLSchema#boolean": _bool_type_def,
+    "https://w3id.org/cwl/salad#null": _null_type_def,
+    "https://w3id.org/cwl/salad#Any": _any_type_def,
+    "string": _string_type_def,
+    "int": _int_type_def,
+    "long": _long_type_def,
+    "float": _float_type_def,
+    "double": _float_type_def,
+    "boolean": _bool_type_def,
+    "null": _null_type_def,
+    "Any": _any_type_def,
 }
 
 
 class JavaCodeGen(CodeGenBase):
     def __init__(
-        self, base: str, target: Optional[str], examples: Optional[str]
+        self,
+        base: str,
+        target: Optional[str],
+        examples: Optional[str],
+        package: str,
+        copyright: Optional[str],
     ) -> None:
         super().__init__()
         self.base_uri = base
-        sp = urlsplit(base)
         self.examples = examples
-        self.package = ".".join(
-            list(reversed(sp.netloc.split("."))) + sp.path.strip("/").split("/")
-        )
+        self.package = package
         self.artifact = self.package.split(".")[-1]
-        target = target or "."
-        self.target_dir = target
+        self.copyright = copyright
+        self.target_dir = Path(target or ".").resolve()
         rel_package_dir = self.package.replace(".", "/")
-        self.rel_package_dir = rel_package_dir
-        self.main_src_dir = os.path.join(
-            self.target_dir, "src", "main", "java", rel_package_dir
-        )
-        self.test_src_dir = os.path.join(
-            self.target_dir, "src", "test", "java", rel_package_dir
-        )
-        self.test_resources_dir = os.path.join(
-            self.target_dir, "src", "test", "resources", rel_package_dir
+        self.rel_package_dir = Path(rel_package_dir)
+        self.main_src_dir = self.target_dir / "src" / "main" / "java" / rel_package_dir
+        self.test_src_dir = self.target_dir / "src" / "test" / "java" / rel_package_dir
+        self.test_resources_dir = (
+            self.target_dir / "src" / "test" / "resources" / rel_package_dir
         )
 
     def prologue(self) -> None:
@@ -141,7 +155,7 @@ class JavaCodeGen(CodeGenBase):
 
     @staticmethod
     def property_name(name: str) -> str:
-        avn = schema.avro_name(name)
+        avn = schema.avro_field_name(name)
         return avn
 
     @staticmethod
@@ -150,6 +164,8 @@ class JavaCodeGen(CodeGenBase):
         if avn in ("class", "extends", "abstract", "default", "package"):
             # reserved words
             avn = avn + "_"
+        if avn and avn.startswith("anon."):
+            avn = avn[5:]
         return avn
 
     def interface_name(self, n: str) -> str:
@@ -169,7 +185,7 @@ class JavaCodeGen(CodeGenBase):
         self.current_class = cls
         self.current_class_is_abstract = abstract
         self.current_loader = StringIO()
-        self.current_fieldtypes = {}  # type: Dict[str, TypeDef]
+        self.current_fieldtypes: Dict[str, TypeDef] = {}
         self.current_fields = StringIO()
         interface_doc_str = f"* Auto-generated interface for <I>{classname}</I><BR>"
         if not abstract:
@@ -180,8 +196,9 @@ class JavaCodeGen(CodeGenBase):
             f"* Auto-generated class implementation for <I>{classname}</I><BR>"
         )
         class_doc_str += doc_to_doc_string(doc)
-        with open(os.path.join(self.main_src_dir, f"{cls}.java"), "w") as f:
-
+        target = self.main_src_dir / f"{cls}.java"
+        with open(target, "w") as f:
+            _logger.info("Writing file: %s", target)
             if extends:
                 ext = (
                     "extends "
@@ -191,7 +208,31 @@ class JavaCodeGen(CodeGenBase):
             else:
                 ext = "extends Savable"
             f.write(
-                """package {package};
+                """// Copyright Common Workflow Language project contributors
+"""
+            )
+            if self.copyright:
+                f.write(
+                    """// {copyright}
+""".format(
+                        copyright=self.copyright
+                    )
+                )
+            f.write(
+                """//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//      http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+
+package {package};
 
 import {package}.utils.Savable;
 
@@ -209,9 +250,35 @@ public interface {cls} {ext} {{""".format(
         if self.current_class_is_abstract:
             return
 
-        with open(os.path.join(self.main_src_dir, f"{cls}Impl.java"), "w") as f:
+        target = self.main_src_dir / f"{cls}Impl.java"
+        with open(target, "w") as f:
+            _logger.info("Writing file: %s", target)
             f.write(
-                """package {package};
+                """// Copyright Common Workflow Language project contributors
+"""
+            )
+            if self.copyright:
+                f.write(
+                    """// {copyright}
+""".format(
+                        copyright=self.copyright
+                    )
+                )
+            f.write(
+                """//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//      http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+
+package {package};
 
 import {package}.utils.LoaderInstances;
 import {package}.utils.LoadingOptions;
@@ -227,7 +294,9 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
   private java.util.Map<String, Object> extensionFields_ =
       new java.util.HashMap<String, Object>();
 """.format(
-                    package=self.package, cls=cls, class_doc_str=class_doc_str
+                    package=self.package,
+                    cls=cls,
+                    class_doc_str=class_doc_str,
                 )
             )
         self.current_loader.write(
@@ -267,11 +336,9 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
             )
         )
 
-    def end_class(self, classname, field_names):
-        # type: (str, List[str]) -> None
-        with open(
-            os.path.join(self.main_src_dir, f"{self.current_class}.java"), "a"
-        ) as f:
+    def end_class(self, classname: str, field_names: List[str]) -> None:
+        """Finish this class."""
+        with open(self.main_src_dir / f"{self.current_class}.java", "a") as f:
             f.write(
                 """
 }
@@ -298,9 +365,9 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
             )
 
         self.current_loader.write("""  }""")
-
+        target = self.main_src_dir / f"{self.current_class}Impl.java"
         with open(
-            os.path.join(self.main_src_dir, f"{self.current_class}Impl.java"),
+            target,
             "a",
         ) as f:
             f.write(self.current_fields.getvalue())
@@ -434,10 +501,35 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
         symbols_decl = 'new String[] {{"{}"}}'.format(
             '", "'.join(sym for sym in symbols)
         )
-        enum_path = os.path.join(self.main_src_dir, f"{clazz}.java")
+        enum_path = self.main_src_dir / f"{clazz}.java"
         with open(enum_path, "w") as f:
+            _logger.info("Writing file: %s", enum_path)
             f.write(
-                """package {package};
+                """// Copyright Common Workflow Language project contributors
+"""
+            )
+            if self.copyright:
+                f.write(
+                    """// {copyright}
+""".format(
+                        copyright=self.copyright
+                    )
+                )
+            f.write(
+                """//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//      http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+
+package {package};
 
 import {package}.utils.ValidationException;
 
@@ -507,9 +599,8 @@ public enum {clazz} {{
 """.format(
             fieldname=fieldname, field_doc_str=doc_to_doc_string(doc, indent_level=1)
         )
-        with open(
-            os.path.join(self.main_src_dir, f"{self.current_class}.java"), "a"
-        ) as f:
+        target = self.main_src_dir / f"{self.current_class}.java"
+        with open(target, "a") as f:
             f.write(
                 """
 {getter_doc_str}
@@ -601,6 +692,7 @@ public enum {clazz} {{
         self.declare_field(name, fieldtype, doc, True)
         if optional:
             set_uri = """
+    Boolean __original_is_null = {safename} == null;
     if ({safename} == null) {{
       if (__docRoot != null) {{
         {safename} = java.util.Optional.of(__docRoot);
@@ -608,7 +700,11 @@ public enum {clazz} {{
         {safename} = java.util.Optional.of("_:" + java.util.UUID.randomUUID().toString());
       }}
     }}
-    __baseUri = (String) {safename}.orElse(null);
+    if (__original_is_null) {{
+        __baseUri = __baseUri_;
+    }} else {{
+        __baseUri = (String) {safename}.orElse(null);
+    }}
 """
         else:
             set_uri = """
@@ -635,7 +731,6 @@ public enum {clazz} {{
         vocab_term: bool,
         ref_scope: Optional[int],
     ) -> TypeDef:
-        assert inner is not None
         instance_type = inner.instance_type or "Object"
         return self.declare_type(
             TypeDef(
@@ -659,12 +754,11 @@ public enum {clazz} {{
     def idmap_loader(
         self, field: str, inner: TypeDef, map_subject: str, map_predicate: Optional[str]
     ) -> TypeDef:
-        assert inner is not None
         instance_type = inner.instance_type or "Object"
         return self.declare_type(
             TypeDef(
                 instance_type=instance_type,
-                name="idmap_{}_{}".format(self.safe_name(field), inner.name),
+                name=f"idmap_{self.safe_name(field)}_{inner.name}",
                 init='new IdMapLoader({}, "{}", "{}")'.format(
                     inner.name, map_subject, map_predicate
                 ),
@@ -699,7 +793,7 @@ public enum {clazz} {{
         pd = pd + " for parsing documents corresponding to the "
         pd = pd + str(self.base_uri) + " schema."
 
-        template_vars = dict(
+        template_vars: MutableMapping[str, str] = dict(
             base_uri=self.base_uri,
             package=self.package,
             group_id=self.package,
@@ -709,49 +803,38 @@ public enum {clazz} {{
             project_description=pd,
             license_name="Apache License, Version 2.0",
             license_url="https://www.apache.org/licenses/LICENSE-2.0.txt",
-        )  # type: MutableMapping[str, str]
+        )
 
-        def template_from_resource(resource):
-            # type: (str) -> string.Template
+        def template_from_resource(resource: str) -> string.Template:
             template_str = pkg_resources.resource_string(
                 __name__, f"java/{resource}"
             ).decode("utf-8")
             template = string.Template(template_str)
             return template
 
-        def expand_resource_template_to(resource, path):
-            # type: (str, str) -> None
+        def expand_resource_template_to(resource: str, path: Path) -> None:
             template = template_from_resource(resource)
             src = template.safe_substitute(template_vars)
             _ensure_directory_and_write(path, src)
 
-        expand_resource_template_to("pom.xml", os.path.join(self.target_dir, "pom.xml"))
-        expand_resource_template_to(
-            "gitignore", os.path.join(self.target_dir, ".gitignore")
-        )
-        expand_resource_template_to(
-            "package.html", os.path.join(self.main_src_dir, "package.html")
-        )
+        expand_resource_template_to("pom.xml", self.target_dir / "pom.xml")
+        expand_resource_template_to("gitignore", self.target_dir / ".gitignore")
+        expand_resource_template_to("package.html", self.main_src_dir / "package.html")
         expand_resource_template_to(
             "overview.html",
-            os.path.join(self.target_dir, "src", "main", "javadoc", "overview.html"),
+            self.target_dir / "src" / "main" / "javadoc" / "overview.html",
         )
         expand_resource_template_to(
             "MANIFEST.MF",
-            os.path.join(
-                self.target_dir, "src", "main", "resources", "META-INF", "MANIFEST.MF"
-            ),
-        )
-        expand_resource_template_to(
-            "README.md",
-            os.path.join(self.target_dir, "README.md"),
+            self.target_dir / "src" / "main" / "resources" / "META-INF" / "MANIFEST.MF",
         )
+        expand_resource_template_to("README.md", self.target_dir / "README.md")
 
         vocab = ""
         rvocab = ""
         for k in sorted(self.vocab.keys()):
-            vocab += """    vocab.put("{}", "{}");\n""".format(k, self.vocab[k])
-            rvocab += """    rvocab.put("{}", "{}");\n""".format(self.vocab[k], k)
+            vocab += f"""    vocab.put("{k}", "{self.vocab[k]}");\n"""
+            rvocab += f"""    rvocab.put("{self.vocab[k]}", "{k}");\n"""
 
         loader_instances = ""
         for _, collected_type in self.collected_types.items():
@@ -762,7 +845,7 @@ public enum {clazz} {{
         example_tests = ""
         if self.examples:
             _safe_makedirs(self.test_resources_dir)
-            utils_resources = os.path.join(self.test_resources_dir, "utils")
+            utils_resources = self.test_resources_dir / "utils"
             if os.path.exists(utils_resources):
                 shutil.rmtree(utils_resources)
             shutil.copytree(self.examples, utils_resources)
@@ -772,12 +855,10 @@ public enum {clazz} {{
                     example_tests += """
   @org.junit.Test
   public void test{basename}ByString() throws Exception {{
-    String path = java.nio.file.Paths.get(".").toAbsolutePath().normalize().toString();
-    String baseUri = Uris.fileUri(path) + "/";
     java.net.URL url = getClass().getResource("{example_name}");
     java.nio.file.Path resPath = java.nio.file.Paths.get(url.toURI());
     String yaml = new String(java.nio.file.Files.readAllBytes(resPath), "UTF8");
-    RootLoader.loadDocument(yaml, baseUri);
+    RootLoader.loadDocument(yaml, url.toString());
   }}
 
   @org.junit.Test
@@ -800,7 +881,7 @@ public enum {clazz} {{
                         example_name=example_name,
                     )
 
-        template_args = dict(
+        template_args: MutableMapping[str, str] = dict(
             package=self.package,
             vocab=vocab,
             rvocab=rvocab,
@@ -808,7 +889,7 @@ public enum {clazz} {{
             root_loader_name=root_loader.name,
             root_loader_instance_type=root_loader.instance_type or "Object",
             example_tests=example_tests,
-        )  # type: MutableMapping[str, str]
+        )
 
         util_src_dirs = {
             "main_utils": self.main_src_dir,
@@ -816,7 +897,7 @@ public enum {clazz} {{
         }
         for (util_src, util_target) in util_src_dirs.items():
             for util in pkg_resources.resource_listdir(__name__, f"java/{util_src}"):
-                src_path = os.path.join(util_target, "utils", util)
+                src_path = util_target / "utils" / util
                 src_template = template_from_resource(os.path.join(util_src, util))
                 src = src_template.safe_substitute(template_args)
                 _ensure_directory_and_write(src_path, src)
