@@ -1,4 +1,5 @@
 import logging
+import unicodedata
 from typing import (
     Any,
     Dict,
@@ -17,7 +18,6 @@ import rdflib
 import rdflib.namespace
 from rdflib import Graph, URIRef
 from rdflib.namespace import RDF, RDFS
-
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
 from .exceptions import SchemaException
@@ -116,7 +116,7 @@ def process_type(
                 predicate = recordname
                 recordname = ln
             else:
-                predicate = "{}:{}".format(defaultPrefix, recordname)
+                predicate = f"{defaultPrefix}:{recordname}"
 
         if context.get(recordname, predicate) != predicate:
             raise SchemaException(
@@ -126,7 +126,7 @@ def process_type(
             )
 
         if not recordname:
-            raise SchemaException("Unable to find/derive recordname for {}".format(t))
+            raise SchemaException(f"Unable to find/derive recordname for {t}")
 
         _logger.debug(
             "Adding to context '%s' %s (%s)", recordname, predicate, type(predicate)
@@ -149,7 +149,27 @@ def process_type(
                 v = v["_@id"] if v.get("_@id", "@")[0] != "@" else None
 
             if bool(v):
-                (ns, ln) = rdflib.namespace.split_uri(str(v))
+                try:
+                    (ns, ln) = rdflib.namespace.split_uri(str(v))
+                except ValueError:
+                    # rdflib 5.0.0 compatibility
+                    uri = str(v)
+                    colon_index = str(v).rfind(":")
+
+                    if colon_index < 0:
+                        raise
+                    split_start = rdflib.namespace.SPLIT_START_CATEGORIES
+                    for j in range(-1 - colon_index, len(uri)):
+                        if unicodedata.category(uri[j]) in split_start or uri[j] == "_":
+                            # _ prevents early split, roundtrip not generate
+                            ns = uri[:j]
+                            if not ns:
+                                break
+                            ln = uri[j:]
+                            break
+                    if not ns or not ln:
+                        raise
+
                 if ns[0:-1] in namespaces:
                     propnode = namespaces[ns[0:-1]][ln]
                 else:
@@ -234,7 +254,7 @@ def makerdf(
         doc_url, frg = urldefrag(url)
         if "/" in frg:
             p = frg.split("/")[0]
-            prefixes[p] = "{}#{}/".format(doc_url, p)
+            prefixes[p] = f"{doc_url}#{p}/"
 
     fix_jsonld_ids(wf, idfields)
 
@@ -243,12 +263,18 @@ def makerdf(
     if isinstance(wf, MutableSequence):
         for w in wf:
             w["@context"] = ctx
-            g.parse(data=json_dumps(w), format="json-ld", publicID=str(workflow))
+            g.parse(
+                data=json_dumps(w, default=str),
+                format="json-ld",
+                publicID=str(workflow),
+            )
     elif isinstance(wf, MutableMapping):
         wf["@context"] = ctx
-        g.parse(data=json_dumps(wf), format="json-ld", publicID=str(workflow))
+        g.parse(
+            data=json_dumps(wf, default=str), format="json-ld", publicID=str(workflow)
+        )
     else:
-        raise SchemaException("{} is not a workflow".format(wf))
+        raise SchemaException(f"{wf} is not a workflow")
 
     # Bug in json-ld loader causes @id fields to be added to the graph
     for sub, pred, obj in g.triples((None, URIRef("@id"), None)):
