@@ -4,7 +4,6 @@ import logging
 import os
 import re
 import sys
-from codecs import StreamWriter
 from io import StringIO, TextIOWrapper
 from typing import (
     IO,
@@ -17,19 +16,24 @@ from typing import (
     Set,
     Tuple,
     Union,
-    cast,
 )
 from urllib.parse import urldefrag
 
 import mistune
 
-from . import schema
 from .exceptions import SchemaSaladException, ValidationException
+from .schema import avro_field_name, extend_and_specialize, get_metaschema
 from .utils import add_dictlist, aslist
+from .validate import avro_type_name
 
 _logger = logging.getLogger("salad")
 
 
+def vocab_type_name(url: str) -> str:
+    """Remove the avro namespace, if any."""
+    return avro_type_name(url).split(".")[-1]
+
+
 def has_types(items: Any) -> List[str]:
     r = []  # type: List[str]
     if isinstance(items, MutableMapping):
@@ -50,7 +54,7 @@ def has_types(items: Any) -> List[str]:
 
 def linkto(item: str) -> str:
     frg = urldefrag(item)[1]
-    return "[{}](#{})".format(frg, to_id(frg))
+    return f"[{frg}](#{to_id(frg)})"
 
 
 class MyRenderer(mistune.Renderer):
@@ -160,9 +164,9 @@ def number_headings(toc: ToC, maindoc: str) -> str:
             m = re.match(r"^(#+) (.*)", line)
             if m is not None:
                 group1 = m.group(1)
-                assert group1 is not None
+                assert group1 is not None  # nosec
                 group2 = m.group(2)
-                assert group2 is not None
+                assert group2 is not None  # nosec
                 num = toc.add_entry(len(group1), group2)
                 line = f"{group1} {num} {group2}"
             line = re.sub(r"^(https?://\S+)", r"[\1](\1)", line)
@@ -186,8 +190,8 @@ class RenderType:
     def __init__(
         self,
         toc: ToC,
-        j: List[Dict[str, str]],
-        renderlist: str,
+        j: List[Dict[str, Any]],
+        renderlist: List[str],
         redirects: Dict[str, str],
         primitiveType: str,
     ) -> None:
@@ -218,8 +222,8 @@ class RenderType:
             if t.get("docAfter"):
                 add_dictlist(self.docAfter, t["docAfter"], t["name"])
 
-        metaschema_loader = schema.get_metaschema()[2]
-        alltypes = schema.extend_and_specialize(j, metaschema_loader)
+        metaschema_loader = get_metaschema()[2]
+        alltypes = extend_and_specialize(j, metaschema_loader)
 
         self.typemap = {}  # type: Dict[str, Dict[str, str]]
         self.uses = {}  # type: Dict[str, List[Tuple[str, str]]]
@@ -267,7 +271,7 @@ class RenderType:
         tp: Any,
         redirects: Dict[str, str],
         nbsp: bool = False,
-        jsonldPredicate: Optional[Dict[str, str]] = None,
+        jsonldPredicate: Optional[Union[Dict[str, str], str]] = None,
     ) -> str:
         if isinstance(tp, MutableSequence):
             if nbsp and len(tp) <= 3:
@@ -288,7 +292,10 @@ class RenderType:
                 ar = "array&lt;{}&gt;".format(
                     self.typefmt(tp["items"], redirects, nbsp=True)
                 )
-                if jsonldPredicate is not None and "mapSubject" in jsonldPredicate:
+                if (
+                    isinstance(jsonldPredicate, dict)
+                    and "mapSubject" in jsonldPredicate
+                ):
                     if "mapPredicate" in jsonldPredicate:
                         ar += " | "
                         if len(ar) > 40:
@@ -315,32 +322,32 @@ class RenderType:
                 "https://w3id.org/cwl/salad#record",
                 "https://w3id.org/cwl/salad#enum",
             ):
-                frg = schema.avro_name(tp["name"])
+                frg = vocab_type_name(tp["name"])
                 if tp["name"] in redirects:
                     return """<a href="{}">{}</a>""".format(redirects[tp["name"]], frg)
                 if tp["name"] in self.typemap:
-                    return """<a href="#{}">{}</a>""".format(to_id(frg), frg)
+                    return f"""<a href="#{to_id(frg)}">{frg}</a>"""
                 if (
                     tp["type"] == "https://w3id.org/cwl/salad#enum"
                     and len(tp["symbols"]) == 1
                 ):
                     return "constant value <code>{}</code>".format(
-                        schema.avro_name(tp["symbols"][0])
+                        avro_field_name(tp["symbols"][0])
                     )
                 return frg
             if isinstance(tp["type"], MutableMapping):
                 return self.typefmt(tp["type"], redirects)
         else:
             if str(tp) in redirects:
-                return """<a href="{}">{}</a>""".format(redirects[tp], redirects[tp])
+                return f"""<a href="{redirects[tp]}">{redirects[tp]}</a>"""
             if str(tp) in basicTypes:
                 return """<a href="{}">{}</a>""".format(
-                    self.primitiveType, schema.avro_name(str(tp))
+                    self.primitiveType, vocab_type_name(str(tp))
                 )
             frg2 = urldefrag(tp)[1]
             if frg2 != "":
                 tp = frg2
-            return """<a href="#{}">{}</a>""".format(to_id(tp), tp)
+            return f"""<a href="#{to_id(tp)}">{tp}</a>"""
         raise SchemaSaladException("We should not be here!")
 
     def render_type(self, f: Dict[str, Any], depth: int) -> None:
@@ -404,12 +411,12 @@ class RenderType:
         else:
             doc = ""
 
+        # Save the first line of the first type definition for the
+        # HTML <title> tag
         if self.title is None and f["doc"]:
-            title = f["doc"][0 : f["doc"].index("\n")]
-            if title.startswith("# "):
-                self.title = title[2:]
-            else:
-                self.title = title
+            self.title = f["doc"].partition("\n")[0]
+            if self.title.startswith("# "):
+                self.title = self.title[2:]
 
         if f["type"] == "documentation":
             f["doc"] = number_headings(self.toc, f["doc"])
@@ -443,7 +450,7 @@ class RenderType:
 
                 desc = i["doc"]
 
-                rfrg = schema.avro_name(i["name"])
+                rfrg = avro_field_name(i["name"])
                 tr = """
 <div class="row responsive-table-row">
 <div class="col-xs-3 col-lg-2"><code>{}</code></div>
@@ -472,7 +479,7 @@ class RenderType:
             for e in ex:
                 for i in e.get("symbols", []):
                     doc += "<tr>"
-                    efrg = schema.avro_name(i)
+                    efrg = avro_field_name(i)
                     doc += "<td><code>{}</code></td><td>{}</td>".format(
                         efrg, enumDesc.get(efrg, "")
                     )
@@ -495,8 +502,8 @@ class RenderType:
 
 def avrold_doc(
     j: List[Dict[str, Any]],
-    outdoc: Union[IO[Any], StreamWriter],
-    renderlist: str,
+    outdoc: IO[Any],
+    renderlist: List[str],
     redirects: Dict[str, str],
     brand: str,
     brandlink: str,
@@ -679,7 +686,8 @@ def avrold_doc(
     )
 
 
-def main() -> None:
+def arg_parser() -> argparse.ArgumentParser:
+    """Build the argument parser."""
     parser = argparse.ArgumentParser()
     parser.add_argument("schema")
     parser.add_argument("--only", action="append")
@@ -689,28 +697,53 @@ def main() -> None:
     parser.add_argument("--brandstyle")
     parser.add_argument("--brandinverse", default=False, action="store_true")
     parser.add_argument("--primtype", default="#PrimitiveType")
-
-    args = parser.parse_args()
-
-    makedoc(args)
+    parser.add_argument("--debug", action="store_true")
+    return parser
 
 
-def makedoc(args: argparse.Namespace) -> None:
+def main() -> None:
+    """Shortcut entrypoint."""
+    args = arg_parser().parse_args()
+    if args.debug:
+        _logger.setLevel(logging.DEBUG)
+    makedoc(
+        sys.stdout,
+        args.schema,
+        args.redirect,
+        args.only,
+        args.brand,
+        args.brandlink,
+        args.primtype,
+        args.brandstyle,
+        args.brandinverse,
+    )
 
-    s = []  # type: List[Dict[str, Any]]
-    a = args.schema
-    with open(a, encoding="utf-8") as f:
-        if a.endswith("md"):
+
+def makedoc(
+    stdout: IO[Any],
+    schema: str,
+    redirects: Optional[List[str]] = None,
+    only: Optional[List[str]] = None,
+    brand: Optional[str] = None,
+    brandlink: Optional[str] = None,
+    primtype: Optional[str] = None,
+    brandstyle: Optional[str] = None,
+    brandinverse: Optional[bool] = False,
+) -> None:
+    """Emit HTML representation of a given schema."""
+    s: List[Dict[str, Any]] = []
+    with open(schema, encoding="utf-8") as f:
+        if schema.endswith("md"):
             s.append(
                 {
-                    "name": os.path.splitext(os.path.basename(a))[0],
+                    "name": os.path.splitext(os.path.basename(schema))[0],
                     "type": "documentation",
                     "doc": f.read(),
                 }
             )
         else:
-            uri = "file://" + os.path.abspath(a)
-            metaschema_loader = schema.get_metaschema()[2]
+            uri = "file://" + os.path.abspath(schema)
+            metaschema_loader = get_metaschema()[2]
             j = metaschema_loader.resolve_ref(uri, "")[0]
             if isinstance(j, MutableSequence):
                 s.extend(j)
@@ -719,24 +752,25 @@ def makedoc(args: argparse.Namespace) -> None:
             else:
                 raise ValidationException("Schema must resolve to a list or a dict")
     redirect = {}
-    for r in args.redirect or []:
+    for r in redirects or []:
         redirect[r.split("=")[0]] = r.split("=")[1]
-    renderlist = args.only if args.only else []
-    stdout = (
-        TextIOWrapper(sys.stdout.buffer, encoding="utf-8")
-        if sys.stdout.encoding != "UTF-8"
-        else cast(TextIOWrapper, sys.stdout)
-    )  # type: Union[TextIOWrapper, StreamWriter]
+    renderlist = only or []
+    if hasattr(stdout, "buffer") and getattr(stdout, "encoding", None) != "UTF-8":
+        wrapped_stdout: IO[Any] = TextIOWrapper(
+            stdout.buffer, encoding="utf-8"  # type: ignore[attr-defined]
+        )
+    else:
+        wrapped_stdout = stdout
     avrold_doc(
         s,
-        stdout,
+        wrapped_stdout,
         renderlist,
         redirect,
-        args.brand,
-        args.brandlink,
-        args.primtype,
-        brandstyle=args.brandstyle,
-        brandinverse=args.brandinverse,
+        brand or "",
+        brandlink or "",
+        primtype or "",
+        brandstyle=brandstyle,
+        brandinverse=brandinverse,
     )
 
 
