@@ -1,7 +1,11 @@
+"""Template code used by python_codegen.py."""
 import copy
 import os
+import pathlib
 import re
+import tempfile
 import uuid as _uuid__  # pylint: disable=unused-import # noqa: F401
+from abc import ABC, abstractmethod
 from io import StringIO
 from typing import (
     Any,
@@ -18,40 +22,30 @@ from typing import (
 from urllib.parse import quote, urlsplit, urlunsplit
 from urllib.request import pathname2url
 
-from ruamel import yaml
 from ruamel.yaml.comments import CommentedMap
 
 from schema_salad.exceptions import SchemaSaladException, ValidationException
 from schema_salad.fetcher import DefaultFetcher, Fetcher
 from schema_salad.sourceline import SourceLine, add_lc_filename
+from schema_salad.utils import yaml_no_ts  # requires schema-salad v8.2+
 
-_vocab = {}  # type: Dict[str, str]
-_rvocab = {}  # type: Dict[str, str]
-
-
-class Savable:
-    @classmethod
-    def fromDoc(cls, _doc, baseuri, loadingOptions, docRoot=None):
-        # type: (Any, str, LoadingOptions, Optional[str]) -> Savable
-        pass
-
-    def save(self, top=False, base_url="", relative_uris=True):
-        # type: (bool, str, bool) -> Dict[str, str]
-        pass
+_vocab: Dict[str, str] = {}
+_rvocab: Dict[str, str] = {}
 
 
 class LoadingOptions:
     def __init__(
         self,
-        fetcher=None,  # type: Optional[Fetcher]
-        namespaces=None,  # type: Optional[Dict[str, str]]
-        schemas=None,  # type: Optional[Dict[str, str]]
-        fileuri=None,  # type: Optional[str]
-        copyfrom=None,  # type: Optional[LoadingOptions]
-        original_doc=None,  # type: Optional[Any]
-    ):  # type: (...) -> None
-        self.idx = {}  # type: Dict[str, Dict[str, Any]]
-        self.fileuri = fileuri  # type: Optional[str]
+        fetcher: Optional[Fetcher] = None,
+        namespaces: Optional[Dict[str, str]] = None,
+        schemas: Optional[Dict[str, str]] = None,
+        fileuri: Optional[str] = None,
+        copyfrom: Optional["LoadingOptions"] = None,
+        original_doc: Optional[Any] = None,
+    ) -> None:
+        """Create a LoadingOptions object."""
+        self.idx: Dict[str, Dict[str, Any]] = {}
+        self.fileuri: Optional[str] = fileuri
         self.namespaces = namespaces
         self.schemas = schemas
         self.original_doc = original_doc
@@ -71,25 +65,12 @@ class LoadingOptions:
             from cachecontrol.caches import FileCache
             from cachecontrol.wrapper import CacheControl
 
-            if "HOME" in os.environ:
-                session = CacheControl(
-                    requests.Session(),
-                    cache=FileCache(
-                        os.path.join(os.environ["HOME"], ".cache", "salad")
-                    ),
-                )
-            elif "TMPDIR" in os.environ:
-                session = CacheControl(
-                    requests.Session(),
-                    cache=FileCache(
-                        os.path.join(os.environ["TMPDIR"], ".cache", "salad")
-                    ),
-                )
-            else:
-                session = CacheControl(
-                    requests.Session(), cache=FileCache("/tmp", ".cache", "salad")
-                )
-            self.fetcher = DefaultFetcher({}, session)  # type: Fetcher
+            root = pathlib.Path(os.environ.get("HOME", tempfile.gettempdir()))
+            session = CacheControl(
+                requests.Session(),
+                cache=FileCache(root / ".cache" / "salad"),
+            )
+            self.fetcher: Fetcher = DefaultFetcher({}, session)
         else:
             self.fetcher = fetcher
 
@@ -104,6 +85,29 @@ class LoadingOptions:
                 self.rvocab[v] = k
 
 
+class Savable(ABC):
+    """Mark classes than have a save() and fromDoc() function."""
+
+    @classmethod
+    @abstractmethod
+    def fromDoc(
+        cls,
+        _doc: Any,
+        baseuri: str,
+        loadingOptions: LoadingOptions,
+        docRoot: Optional[str] = None,
+    ) -> "Savable":
+        """Construct this object from the result of yaml.load()."""
+        pass
+
+    @abstractmethod
+    def save(
+        self, top: bool = False, base_url: str = "", relative_uris: bool = True
+    ) -> Dict[str, Any]:
+        """Convert this object to a JSON/YAML friendly dictionary."""
+        pass
+
+
 def load_field(val, fieldtype, baseuri, loadingOptions):
     # type: (Union[str, Dict[str, str]], _Loader, str, LoadingOptions) -> Any
     if isinstance(val, MutableMapping):
@@ -124,15 +128,15 @@ def load_field(val, fieldtype, baseuri, loadingOptions):
     return fieldtype.load(val, baseuri, loadingOptions)
 
 
-save_type = Union[Dict[str, str], List[Union[Dict[str, str], List[Any], None]], None]
+save_type = Union[Dict[str, Any], List[Union[Dict[str, Any], List[Any], None]], None]
 
 
 def save(
-    val,  # type: Optional[Union[Savable, MutableSequence[Savable]]]
-    top=True,  # type: bool
-    base_url="",  # type: str
-    relative_uris=True,  # type: bool
-):  # type: (...) -> save_type
+    val: Optional[Union[Savable, MutableSequence[Savable]]],
+    top: bool = True,
+    base_url: str = "",
+    relative_uris: bool = True,
+) -> save_type:
 
     if isinstance(val, Savable):
         return val.save(top=top, base_url=base_url, relative_uris=relative_uris)
@@ -330,11 +334,32 @@ class _SecondaryDSLLoader(_Loader):
                                 d
                             )
                         )
+                    r.append(new_dict)
 
                 else:
                     raise ValidationException(
                         "Expected a string or sequence of (strings or mappings)."
                     )
+        elif isinstance(doc, MutableMapping):
+            new_dict = {}
+            if "pattern" in doc:
+                new_dict["pattern"] = doc.pop("pattern")
+            else:
+                raise ValidationException(
+                    "Missing pattern in secondaryFiles specification entry: {}".format(
+                        doc
+                    )
+                )
+            new_dict["required"] = doc.pop("required") if "required" in doc else None
+
+            if len(doc):
+                raise ValidationException(
+                    "Unallowed values in secondaryFiles specification entry: {}".format(
+                        doc
+                    )
+                )
+            r.append(new_dict)
+
         elif isinstance(doc, str):
             if doc.endswith("?"):
                 r.append({"pattern": doc[:-1], "required": False})
@@ -449,7 +474,7 @@ class _TypeDSLLoader(_Loader):
         m = self.typeDSLregex.match(doc)
         if m:
             group1 = m.group(1)
-            assert group1 is not None
+            assert group1 is not None  # nosec
             first = expand_url(
                 group1, baseuri, loadingOptions, False, True, self.refScope
             )
@@ -568,7 +593,8 @@ def _document_load_by_url(loader, url, loadingOptions):
     else:
         textIO = StringIO(text)
     textIO.name = str(url)
-    result = yaml.main.round_trip_load(textIO, preserve_quotes=True)
+    yaml = yaml_no_ts()
+    result = yaml.load(textIO)
     add_lc_filename(result, url)
 
     loadingOptions.idx[url] = result
@@ -594,15 +620,22 @@ def file_uri(path, split_frag=False):  # type: (str, bool) -> str
         return f"file://{urlpath}{frag}"
 
 
-def prefix_url(url, namespaces):  # type: (str, Dict[str, str]) -> str
+def prefix_url(url: str, namespaces: Dict[str, str]) -> str:
+    """Expand short forms into full URLs using the given namespace dictionary."""
     for k, v in namespaces.items():
         if url.startswith(v):
             return k + ":" + url[len(v) :]
     return url
 
 
-def save_relative_uri(uri, base_url, scoped_id, ref_scope, relative_uris):
-    # type: (str, str, bool, Optional[int], bool) -> Union[str, List[str]]
+def save_relative_uri(
+    uri: Any,
+    base_url: str,
+    scoped_id: bool,
+    ref_scope: Optional[int],
+    relative_uris: bool,
+) -> Any:
+    """Convert any URI to a relative one, obeying the scoping rules."""
     if not relative_uris or uri == base_url:
         return uri
     if isinstance(uri, MutableSequence):
