@@ -1,8 +1,11 @@
 import copy
 import logging
 import os
+import pathlib
 import re
-import xml.sax
+import tempfile
+import urllib
+import xml.sax  # nosec
 from io import StringIO
 from typing import (
     Any,
@@ -13,7 +16,6 @@ from typing import (
     MutableSequence,
     Optional,
     Set,
-    TypeVar,
     Union,
     cast,
 )
@@ -24,9 +26,8 @@ from cachecontrol.wrapper import CacheControl
 from rdflib.graph import Graph
 from rdflib.namespace import OWL, RDF, RDFS
 from rdflib.plugins.parsers.notation3 import BadSyntax
-from ruamel import yaml
 from ruamel.yaml.comments import CommentedMap, CommentedSeq, LineCol
-from six.moves import urllib
+from ruamel.yaml.error import MarkedYAMLError
 
 from .exceptions import SchemaSaladException, ValidationException
 from .fetcher import DefaultFetcher
@@ -42,6 +43,7 @@ from .utils import (
     ResolveType,
     aslist,
     onWindows,
+    yaml_no_ts,
 )
 
 _logger = logging.getLogger("salad")
@@ -77,7 +79,8 @@ def uri_file_path(url: str) -> str:
     raise ValidationException(f"Not a file URI: {url}")
 
 
-def to_validation_exception(e: yaml.error.MarkedYAMLError) -> ValidationException:
+def to_validation_exception(e: MarkedYAMLError) -> ValidationException:
+    """Convert ruamel.yaml exception to our type."""
     fname_regex = re.compile(r"^file://" + re.escape(os.getcwd()) + "/")
 
     exc = ValidationException(e.problem)
@@ -169,25 +172,11 @@ class Loader:
             if doc_cache is False:
                 self.session = requests.Session()
             elif doc_cache is True:
-                if "HOME" in os.environ:
-                    self.session = CacheControl(
-                        requests.Session(),
-                        cache=FileCache(
-                            os.path.join(os.environ["HOME"], ".cache", "salad")
-                        ),
-                    )
-                elif "TMP" in os.environ:
-                    self.session = CacheControl(
-                        requests.Session(),
-                        cache=FileCache(
-                            os.path.join(os.environ["TMP"], ".cache", "salad")
-                        ),
-                    )
-                else:
-                    self.session = CacheControl(
-                        requests.Session(),
-                        cache=FileCache(os.path.join("/tmp", ".cache", "salad")),
-                    )
+                root = pathlib.Path(os.environ.get("HOME", tempfile.gettempdir()))
+                self.session = CacheControl(
+                    requests.Session(),
+                    cache=FileCache(root / ".cache" / "salad"),
+                )
             elif isinstance(doc_cache, str):
                 self.session = CacheControl(
                     requests.Session(), cache=FileCache(doc_cache)
@@ -308,11 +297,7 @@ class Loader:
                             )
                             self.graph += self.cache[fetchurl]
                             break
-                        except xml.sax.SAXParseException:
-                            pass
-                        except TypeError:
-                            pass
-                        except BadSyntax:
+                        except (xml.sax.SAXParseException, TypeError, BadSyntax):
                             pass
             except Exception as e:
                 _logger.warning(
@@ -413,6 +398,7 @@ class Loader:
         base_url: Optional[str] = None,
         checklinks: bool = True,
         strict_foreign_properties: bool = False,
+        content_types: Optional[List[str]] = None,  # Expected content-types
     ) -> ResolvedRefType:
 
         lref = ref
@@ -468,7 +454,7 @@ class Loader:
 
         if not isinstance(lref, str):
             raise ValidationException(
-                "Expected CommentedMap or string, got {}: `{}`".format(type(lref), lref)
+                f"Expected CommentedMap or string, got {type(lref)}: `{lref}`"
             )
 
         if isinstance(lref, str) and os.sep == "\\":
@@ -529,7 +515,9 @@ class Loader:
                 raise ValidationException(
                     f"Reference `#{frg}` not found in file `{doc_url}`.", sl
                 )
-            doc = self.fetch(doc_url, inject_ids=(not mixin))
+            doc = self.fetch(
+                doc_url, inject_ids=(not mixin), content_types=content_types
+            )
 
         # Recursively expand urls and resolve directives
         if bool(mixin):
@@ -637,7 +625,7 @@ class Loader:
         if not m:
             return t
         first = m.group(1)
-        assert first
+        assert first  # nosec
         second = third = None
         if bool(m.group(2)):
             second = CommentedMap((("type", "array"), ("items", first)))
@@ -697,7 +685,11 @@ class Loader:
                 datum2 = datum = document[d]
                 if isinstance(datum, str):
                     datum2 = self._apply_dsl(
-                        datum, d, loader, document.lc.data[d], document.lc.filename
+                        datum,
+                        d,
+                        loader,
+                        document.lc.data[d] if document.lc.data else document.lc,
+                        getattr(document.lc, "filename", ""),
                     )
                 elif isinstance(datum, CommentedSeq):
                     datum2 = CommentedSeq()
@@ -776,7 +768,7 @@ class Loader:
                             v, base_url, scoped_id=True
                         )
                         if document[identifer][n] not in loader.idx:
-                            loader.idx[document[identifer][n]] = v
+                            loader.idx[cast(str, document[identifer][n])] = v
 
     def _normalize_fields(self, document: CommentedMap, loader: "Loader") -> None:
         # Normalize fields which are prefixed or full URIn to vocabulary terms
@@ -944,7 +936,7 @@ class Loader:
                             for j in range(len(document) + llen, i + llen, -1):
                                 document.lc.data[j - 1] = document.lc.data[j - llen]
                             for item in l:
-                                cast(CommentedSeq, document).insert(i, item)
+                                document.insert(i, item)
                                 document.lc.data[i] = lc
                                 i += 1
                         else:
@@ -976,19 +968,25 @@ class Loader:
 
         return document, metadata
 
-    def fetch(self, url: str, inject_ids: bool = True) -> IdxResultType:
+    def fetch(
+        self,
+        url: str,
+        inject_ids: bool = True,
+        content_types: Optional[List[str]] = None,
+    ) -> IdxResultType:
         if url in self.idx:
             return self.idx[url]
         try:
-            text = self.fetch_text(url)
+            text = self.fetch_text(url, content_types=content_types)
             if isinstance(text, bytes):
                 textIO = StringIO(text.decode("utf-8"))
 
             else:
                 textIO = StringIO(text)
             textIO.name = str(url)
-            attachments = yaml.main.round_trip_load_all(textIO, preserve_quotes=True)
-            result = cast(CommentedMap, next(attachments))
+            yaml = yaml_no_ts()
+            attachments = yaml.load_all(textIO)
+            result = cast(Union[CommentedSeq, CommentedMap], next(attachments))
 
             if self.allow_attachments is not None and self.allow_attachments(result):
                 i = 1
@@ -996,15 +994,18 @@ class Loader:
                     self.idx[f"{url}#attachment-{i}"] = a
                     i += 1
             add_lc_filename(result, url)
-        except yaml.error.MarkedYAMLError as e:
+        except MarkedYAMLError as e:
             raise to_validation_exception(e) from e
         if isinstance(result, CommentedMap) and inject_ids and bool(self.identifiers):
+            missing_identifier = True
             for identifier in self.identifiers:
-                if identifier not in result:
-                    result[identifier] = url
-                self.idx[
-                    self.expand_url(result[identifier], url, scoped_id=True)
-                ] = result
+                if identifier in result:
+                    missing_identifier = False
+                    self.idx[
+                        self.expand_url(result[identifier], url, scoped_id=True)
+                    ] = result
+            if missing_identifier:
+                result[self.identifiers[0]] = url
         self.idx[url] = result
         return result
 
@@ -1176,7 +1177,7 @@ class Loader:
                     if docid2 is not None:
                         errors.append(
                             ValidationException(
-                                "checking object `{}`".format(relname(docid2)), sl, [v]
+                                f"checking object `{relname(docid2)}`", sl, [v]
                             )
                         )
                     else:
@@ -1194,10 +1195,9 @@ class Loader:
         return
 
 
-D = TypeVar("D", CommentedMap, ContextType)
-
-
-def _copy_dict_without_key(from_dict: D, filtered_key: str) -> D:
+def _copy_dict_without_key(
+    from_dict: Union[CommentedMap, ContextType], filtered_key: str
+) -> CommentedMap:
     new_dict = CommentedMap(from_dict.items())
     if filtered_key in new_dict:
         del new_dict[filtered_key]
