@@ -17,10 +17,9 @@ from typing import (
     Union,
     cast,
 )
-from urllib.parse import urldefrag, urlparse
+from urllib.parse import urlparse
 
 from pkg_resources import resource_stream
-from ruamel import yaml
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
 from schema_salad.utils import (
@@ -31,6 +30,7 @@ from schema_salad.utils import (
     convert_to_dict,
     flatten,
     json_dumps,
+    yaml_no_ts,
 )
 
 from . import _logger, jsonld_context, ref_resolver, validate
@@ -183,7 +183,8 @@ def get_metaschema() -> Tuple[Names, List[Dict[str, str]], Loader]:
     with resource_stream("schema_salad", "metaschema/metaschema.yml") as stream:
         loader.cache["https://w3id.org/cwl/salad"] = stream.read().decode("UTF-8")
 
-    j = yaml.main.round_trip_load(loader.cache["https://w3id.org/cwl/salad"])
+    yaml = yaml_no_ts()
+    j = yaml.load(loader.cache["https://w3id.org/cwl/salad"])
     add_lc_filename(j, "metaschema.yml")
     j2 = loader.resolve_all(j, saladp)[0]
 
@@ -191,7 +192,7 @@ def get_metaschema() -> Tuple[Names, List[Dict[str, str]], Loader]:
         _logger.error("%s", j2)
         raise SchemaParseException(f"Not a list: {j2}")
     else:
-        sch_obj = make_avro(j2, loader)
+        sch_obj = make_avro(j2, loader, loader.vocab)
     try:
         sch_names = make_avro_schema_from_avro(sch_obj)
     except SchemaParseException:
@@ -257,7 +258,7 @@ def load_schema(
 
     # Make the Avro validation that will be used to validate the target
     # document
-    avsc_names = make_avro_schema(schema_doc, document_loader)
+    avsc_names = make_avro_schema(schema_doc, document_loader, metaschema_loader.vocab)
 
     return document_loader, avsc_names, schema_metadata, metaschema_loader
 
@@ -349,6 +350,7 @@ def validate_doc(
                 raise_ex=False,
                 skip_foreign_properties=loader.skip_schemas,
                 strict_foreign_properties=strict_foreign_properties,
+                vocab=loader.vocab,
             )
             if success:
                 break
@@ -371,17 +373,20 @@ def validate_doc(
                         raise_ex=True,
                         skip_foreign_properties=loader.skip_schemas,
                         strict_foreign_properties=strict_foreign_properties,
+                        vocab=loader.vocab,
                     )
                 except ClassValidationException as exc1:
                     errors = [
                         ClassValidationException(
-                            f"tried `{name}` but", sourceline, [exc1]
+                            f"tried `{validate.friendly(name)}` but", sourceline, [exc1]
                         )
                     ]
                     break
                 except ValidationException as exc2:
                     errors.append(
-                        ValidationException(f"tried `{name}` but", sourceline, [exc2])
+                        ValidationException(
+                            f"tried `{validate.friendly(name)}` but", sourceline, [exc2]
+                        )
                     )
 
             objerr = "Invalid"
@@ -409,7 +414,9 @@ def get_anon_name(
     if rec["type"] in ("enum", saladp + "enum"):
         for sym in rec["symbols"]:
             anon_name += sym
-        return "enum_" + hashlib.sha1(anon_name.encode("UTF-8")).hexdigest()
+        return (
+            "anon.enum_" + hashlib.sha1(anon_name.encode("UTF-8")).hexdigest()  # nosec
+        )
     if rec["type"] in ("record", saladp + "record"):
         for field in rec["fields"]:
             if isinstance(field, Mapping):
@@ -420,7 +427,7 @@ def get_anon_name(
                         field
                     )
                 )
-        return "record_" + hashlib.sha1(anon_name.encode("UTF-8")).hexdigest()
+        return "record_" + hashlib.sha1(anon_name.encode("UTF-8")).hexdigest()  # nosec
     if rec["type"] in ("array", saladp + "array"):
         return ""
     raise ValidationException("Expected enum or record, was {}".format(rec["type"]))
@@ -434,8 +441,7 @@ def replace_type(
     find_embeds: bool = True,
     deepen: bool = True,
 ) -> Any:
-    """ Go through and replace types in the 'spec' mapping"""
-
+    """Go through and replace types in the 'spec' mapping."""
     if isinstance(items, MutableMapping):
         # recursively check these fields for types to replace
         if items.get("type") in ("record", "enum") and items.get("name"):
@@ -489,7 +495,7 @@ def replace_type(
     return items
 
 
-def avro_name(url: str) -> str:
+def avro_field_name(url: str) -> str:
     """
     Turn a URL into an Avro-safe name.
 
@@ -498,12 +504,10 @@ def avro_name(url: str) -> str:
     Extract either the last part of the URL fragment past the slash, otherwise
     the whole fragment.
     """
-    frg = urldefrag(url)[1]
-    if frg != "":
-        if "/" in frg:
-            return frg[frg.rindex("/") + 1 :]
-        return frg
-    return url
+    d = urlparse(url)
+    if d.fragment:
+        return d.fragment.split("/")[-1]
+    return d.path.split("/")[-1]
 
 
 Avro = TypeVar("Avro", MutableMapping[str, Any], MutableSequence[Any], str)
@@ -514,15 +518,25 @@ def make_valid_avro(
     alltypes: Dict[str, Dict[str, Any]],
     found: Set[str],
     union: bool = False,
+    fielddef: bool = False,
+    vocab: Optional[Dict[str, str]] = None,
 ) -> Union[
     Avro, MutableMapping[str, str], str, List[Union[Any, MutableMapping[str, str], str]]
 ]:
     """Convert our schema to be more avro like."""
+
+    if vocab is None:
+        _, _, metaschema_loader = get_metaschema()
+        vocab = metaschema_loader.vocab
+
     # Possibly could be integrated into our fork of avro/schema.py?
     if isinstance(items, MutableMapping):
         avro = copy.copy(items)
-        if avro.get("name") and avro.get("inVocab", True):
-            avro["name"] = avro_name(avro["name"])
+        if avro.get("name"):
+            if fielddef:
+                avro["name"] = avro_field_name(avro["name"])
+            else:
+                avro["name"] = validate.avro_type_name(avro["name"])
 
         if "type" in avro and avro["type"] in (
             saladp + "record",
@@ -537,19 +551,35 @@ def make_valid_avro(
             found.add(avro["name"])
         for field in ("type", "items", "values", "fields"):
             if field in avro:
-                avro[field] = make_valid_avro(avro[field], alltypes, found, union=True)
+                avro[field] = make_valid_avro(
+                    avro[field],
+                    alltypes,
+                    found,
+                    union=True,
+                    fielddef=(field == "fields"),
+                    vocab=vocab,
+                )
         if "symbols" in avro:
-            avro["symbols"] = [avro_name(sym) for sym in avro["symbols"]]
+            avro["symbols"] = [avro_field_name(sym) for sym in avro["symbols"]]
         return avro
     if isinstance(items, MutableSequence):
         ret = []
         for i in items:
-            ret.append(make_valid_avro(i, alltypes, found, union=union))
+            ret.append(
+                make_valid_avro(
+                    i, alltypes, found, union=union, fielddef=fielddef, vocab=vocab
+                )
+            )
         return ret
     if union and isinstance(items, str):
-        if items in alltypes and avro_name(items) not in found:
-            return make_valid_avro(alltypes[items], alltypes, found, union=union)
-        return avro_name(items)
+        if items in alltypes and validate.avro_type_name(items) not in found:
+            return make_valid_avro(
+                alltypes[items], alltypes, found, union=union, vocab=vocab
+            )
+        if items in vocab:
+            return validate.avro_type_name(vocab[items])
+        else:
+            return validate.avro_type_name(items)
     else:
         return items
 
@@ -647,7 +677,7 @@ def extend_and_specialize(
             for ex in aslist(result["extends"]):
                 if ex_types[ex].get("abstract"):
                     add_dictlist(extended_by, ex, ex_types[result["name"]])
-                    add_dictlist(extended_by, avro_name(ex), ex_types[ex])
+                    add_dictlist(extended_by, validate.avro_type_name(ex), ex_types[ex])
 
     for result in results:
         if result.get("abstract") and result["name"] not in extended_by:
@@ -667,6 +697,7 @@ def extend_and_specialize(
 def make_avro(
     i: List[Dict[str, Any]],
     loader: Loader,
+    metaschema_vocab: Optional[Dict[str, str]] = None,
 ) -> List[Any]:
 
     j = extend_and_specialize(i, loader)
@@ -674,20 +705,20 @@ def make_avro(
     name_dict = {}  # type: Dict[str, Dict[str, Any]]
     for entry in j:
         name_dict[entry["name"]] = entry
-    avro = make_valid_avro(j, name_dict, set())
+
+    avro = make_valid_avro(j, name_dict, set(), vocab=metaschema_vocab)
 
     return [
         t
         for t in avro
         if isinstance(t, MutableMapping)
         and not t.get("abstract")
-        and t.get("type") != "documentation"
+        and t.get("type") != "org.w3id.cwl.salad.documentation"
     ]
 
 
 def make_avro_schema(
-    i: List[Any],
-    loader: Loader,
+    i: List[Any], loader: Loader, metaschema_vocab: Optional[Dict[str, str]] = None
 ) -> Names:
     """
     All in one convenience function.
@@ -696,7 +727,7 @@ def make_avro_schema(
     the intermediate result for diagnostic output.
     """
     names = Names()
-    avro = make_avro(i, loader)
+    avro = make_avro(i, loader, metaschema_vocab)
     make_avsc_object(convert_to_dict(avro), names)
     return names
 
@@ -730,7 +761,7 @@ def print_inheritance(doc: List[Dict[str, Any]], stream: IO[Any]) -> None:
             stream.write(f'"{name}" [shape={shape} label="{label}"];\n')
             if "extends" in entry:
                 for target in aslist(entry["extends"]):
-                    stream.write('"{}" -> "{}";\n'.format(shortname(target), name))
+                    stream.write(f'"{shortname(target)}" -> "{name}";\n')
     stream.write("}\n")
 
 
