@@ -1,6 +1,10 @@
+"""Test examples."""
+import datetime
 import os
+from io import StringIO
+from typing import Any, Dict, cast
 
-import ruamel.yaml
+from pytest import CaptureFixture
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
 import schema_salad.main
@@ -8,7 +12,7 @@ import schema_salad.schema
 from schema_salad.jsonld_context import makerdf
 from schema_salad.ref_resolver import Loader, file_uri, uri_file_path
 from schema_salad.sourceline import SourceLine, cmap
-from schema_salad.utils import ContextType
+from schema_salad.utils import ContextType, stdout, yaml_no_ts
 
 from .util import get_data
 
@@ -43,6 +47,90 @@ def test_self_validate() -> None:
     assert 0 == schema_salad.main.main(argsl=[path, path])
 
 
+def test_print_rdf(capfdbinary: CaptureFixture[bytes]) -> None:
+    """Test --print-rdf."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    document_path = get_data("tests/test_real_cwl/bio-cwl-tools/bamtools_stats.cwl")
+    assert schema_path and document_path
+    assert 0 == schema_salad.main.main(
+        argsl=["--print-rdf", schema_path, document_path]
+    )
+    captured = capfdbinary.readouterr()
+    assert captured.out
+
+
+def test_print_pre_schema(capfdbinary: CaptureFixture[bytes]) -> None:
+    """Test --print-pre only schema."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    assert schema_path
+    assert 0 == schema_salad.main.main(argsl=["--print-pre", schema_path])
+    captured = capfdbinary.readouterr()
+    assert captured.out
+
+
+def test_print_pre(capfdbinary: CaptureFixture[bytes]) -> None:
+    """Test --print-pre."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    document_path = get_data("tests/test_real_cwl/bio-cwl-tools/bamtools_stats.cwl")
+    assert schema_path and document_path
+    assert 0 == schema_salad.main.main(
+        argsl=["--print-pre", schema_path, document_path]
+    )
+    captured = capfdbinary.readouterr()
+    assert captured.out
+
+
+def test_print_schema_index(capfdbinary: CaptureFixture[bytes]) -> None:
+    """Test --print-index only with a schema."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    assert schema_path
+    assert 0 == schema_salad.main.main(argsl=["--print-index", schema_path])
+    captured = capfdbinary.readouterr()
+    assert captured.out
+
+
+def test_print_index(capfdbinary: CaptureFixture[bytes]) -> None:
+    """Test --print-index."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    document_path = get_data("tests/test_real_cwl/bio-cwl-tools/bamtools_stats.cwl")
+    assert schema_path and document_path
+    assert 0 == schema_salad.main.main(
+        argsl=["--print-index", schema_path, document_path]
+    )
+    captured = capfdbinary.readouterr()
+    assert captured.out
+
+
+def test_print_schema_metadata(capfdbinary: CaptureFixture[bytes]) -> None:
+    """Test --print-metadata only for a schema."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    assert schema_path
+    assert 0 == schema_salad.main.main(argsl=["--print-metadata", schema_path])
+    captured = capfdbinary.readouterr()
+    assert captured.out
+
+
+def test_print_metadata(capfdbinary: CaptureFixture[bytes]) -> None:
+    """Test --print-metadata."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    document_path = get_data("tests/test_real_cwl/bio-cwl-tools/bamtools_stats.cwl")
+    assert schema_path and document_path
+    assert 0 == schema_salad.main.main(
+        argsl=["--print-metadata", schema_path, document_path]
+    )
+    captured = capfdbinary.readouterr()
+    assert captured.out
+
+
+def test_schema_salad_doc_oneline_doc() -> None:
+    """Test schema-salad-doc when the 1st type has only a single doc line."""
+    schema_path = get_data("tests/test_schema/one_line_primary_doc.yml")
+    assert schema_path
+    stdout = StringIO()
+    schema_salad.makedoc.makedoc(stdout, schema_path)
+    assert "<title>Schema</title>" in stdout.getvalue()
+
+
 def test_avro_regression() -> None:
     path = get_data("tests/Process.yml")
     assert path
@@ -199,19 +287,18 @@ def test_examples() -> None:
         ldr, _, _, _ = schema_salad.schema.load_schema(path)
         path2 = get_data(f"metaschema/{a}_src.yml")
         assert path2
+        yaml = yaml_no_ts()
         with open(path2) as src_fp:
-            src = ldr.resolve_all(
-                ruamel.yaml.main.round_trip_load(src_fp), "", checklinks=False
-            )[0]
+            src = ldr.resolve_all(yaml.load(src_fp), "", checklinks=False)[0]
         path3 = get_data(f"metaschema/{a}_proc.yml")
         assert path3
         with open(path3) as src_proc:
-            proc = ruamel.yaml.main.safe_load(src_proc)
+            proc = yaml.load(src_proc)
         assert proc == src
 
 
 def test_yaml_float_test() -> None:
-    assert ruamel.yaml.main.safe_load("float-test: 2e-10")["float-test"] == 2e-10
+    assert yaml_no_ts().load("float-test: 2e-10")["float-test"] == 2e-10
 
 
 def test_typedsl_ref() -> None:
@@ -274,7 +361,7 @@ def test_scoped_id() -> None:
     } == ra
 
     g = makerdf(None, ra, ctx)
-    print(g.serialize(format="n3"))
+    g.serialize(destination=stdout(), format="n3")
 
     ra, _ = ldr.resolve_all(
         cmap({"location": "foo", "bar": {"location": "baz"}}),
@@ -287,7 +374,7 @@ def test_scoped_id() -> None:
     } == ra
 
     g = makerdf(None, ra, ctx)
-    print(g.serialize(format="n3"))
+    g.serialize(destination=stdout(), format="n3")
 
     ra, _ = ldr.resolve_all(
         cmap({"id": "foo", "bar": {"location": "baz"}}),
@@ -300,7 +387,7 @@ def test_scoped_id() -> None:
     } == ra
 
     g = makerdf(None, ra, ctx)
-    print(g.serialize(format="n3"))
+    g.serialize(destination=stdout(), format="n3")
 
     ra, _ = ldr.resolve_all(
         cmap({"location": "foo", "bar": {"id": "baz"}}),
@@ -313,7 +400,53 @@ def test_scoped_id() -> None:
     } == ra
 
     g = makerdf(None, ra, ctx)
-    print(g.serialize(format="n3"))
+    g.serialize(destination=stdout(), format="n3")
+
+
+def test_rdf_datetime() -> None:
+    """Affirm that datetime objects can be serialized in makerdf()."""
+    ldr = Loader({})
+    ctx: ContextType = {
+        "id": "@id",
+        "location": {"@id": "@id", "@type": "@id"},
+        "bar": "http://example.com/bar",
+        "ex": "http://example.com/",
+    }
+    ldr.add_context(ctx)
+
+    ra: CommentedMap = cast(
+        CommentedMap,
+        ldr.resolve_all(
+            cmap(
+                {
+                    "id": "foo",
+                    "bar": {"id": "baz"},
+                }
+            ),
+            "http://example.com",
+        )[0],
+    )
+    ra["s:dateCreated"] = datetime.datetime(2020, 10, 8)
+
+    g = makerdf(None, ra, ctx)
+    g.serialize(destination=stdout(), format="n3")
+    g2 = makerdf(None, CommentedSeq([ra]), ctx)
+    g2.serialize(destination=stdout(), format="n3")
+
+
+def test_yaml_datetime() -> None:
+    """Affirm that yaml_no_ts prevents the creation of datetime objects."""
+    example: Dict[str, Any] = {
+        "id": "foo",
+        "bar": {"id": "baz"},
+    }
+    example["s:dateCreated"] = datetime.datetime(2020, 10, 8)
+    yaml = yaml_no_ts()
+    stream = StringIO()
+    yaml.dump(example, stream)
+    stream2 = StringIO(stream.getvalue())
+    example2 = yaml.load(stream2)
+    assert isinstance(example2["s:dateCreated"], str)
 
 
 def test_subscoped_id() -> None:
