@@ -1,5 +1,7 @@
 import json
 import os
+import sys
+from io import BufferedWriter
 from typing import (
     IO,
     TYPE_CHECKING,
@@ -16,8 +18,9 @@ from typing import (
 
 import requests
 from rdflib.graph import Graph
-
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
+from ruamel.yaml.constructor import RoundTripConstructor
+from ruamel.yaml.main import YAML
 
 if TYPE_CHECKING:
     from .fetcher import Fetcher
@@ -99,7 +102,7 @@ def json_dump(
     fp,  # type: IO[str]
     **kwargs  # type: Any
 ):  # type: (...) -> None
-    """ Force use of unicode. """
+    """Force use of unicode."""
     json.dump(convert_to_dict(obj), fp, **kwargs)
 
 
@@ -107,5 +110,33 @@ def json_dumps(
     obj,  # type: Any
     **kwargs  # type: Any
 ):  # type: (...) -> str
-    """ Force use of unicode. """
+    """Force use of unicode."""
     return json.dumps(convert_to_dict(obj), **kwargs)
+
+
+def stdout() -> BufferedWriter:
+    """Build a replacement for sys.stdout that allow for writing binary data."""
+    return os.fdopen(sys.stdout.fileno(), "wb", closefd=False)
+
+
+class _RoundTripNoTimeStampConstructor(RoundTripConstructor):
+    def construct_yaml_timestamp(self: Any, node: Any, values: Any = None) -> Any:
+        return node.value
+
+
+_RoundTripNoTimeStampConstructor.add_constructor(
+    "tag:yaml.org,2002:timestamp",
+    _RoundTripNoTimeStampConstructor.construct_yaml_timestamp,
+)
+
+
+def yaml_no_ts() -> YAML:
+    """
+    Get a YAML loader that won't parse timestamps into datetime objects.
+
+    Such datetime objects can't be easily dumped into JSON.
+    """
+    yaml = YAML(typ="rt")
+    yaml.preserve_quotes = True  # type: ignore
+    yaml.Constructor = _RoundTripNoTimeStampConstructor
+    return yaml
